@@ -382,10 +382,17 @@ predict.kmeans <- function(object, data, ...)
 
 genPredictKmeans <- function(dataset)
 {
+  
   # 081227 Generate a command to obtain the prediction results when
   # applying the model to new data.
-  
-  return(sprintf("crs$pr <- predict(crs$kmeans, %s)", dataset))
+  print(not.null(crs$xdf))
+  print(crs$xdf)
+  if(not.null(crs$xdf)){
+    class(crs$kmeans) <- c("rxKmeans","kmeans")
+    return(sprintf("crs$pr <- predict(crs$kmeans, %s)", rxDataStep(inData = eval(parse(text = dataset)))))
+  }
+  else 
+    return(sprintf("crs$pr <- predict(crs$kmeans, %s)", dataset))
 }
 
 genResponseKmeans <- function(dataset)
