@@ -1,6 +1,6 @@
 # Gnome R Data Miner: GNOME interface to R for Data Mining
 #
-# Time-stamp: <2015-05-17 08:57:27 gjw>
+# Time-stamp: <2017-07-10 20:47:16 Graham Williams>
 #
 # MODEL TAB
 #
@@ -56,7 +56,7 @@ on_boost_radiobutton_toggled <- function(button)
   if (button$getActive())
   {
     ## crv$MODEL$setCurrentPage(crv$MODEL.GBM.TAB)
-    crv$MODEL$setCurrentPage(crv$MODEL.ADA.TAB)
+     crv$MODEL$setCurrentPage(crv$MODEL.ADA.TAB)
     ## setTextview("confusion_textview")
     resetTextviews("confusion_textview")
   }
@@ -183,6 +183,52 @@ on_model_tree_ctree_radiobutton_toggled <- function(button)
   }
 }
 
+#-----------------------------------------------------------------------
+# Model -> Boost
+#
+# Set the model builder label appropriately.
+
+setADAOptions <- function(mtype)
+{
+  theWidget("ada_ntree_label")$setSensitive(mtype=="ada")
+  theWidget("ada_ntree_spinbutton")$setSensitive(mtype=="ada")
+  theWidget("ada_min_split_label")$setSensitive(mtype=="ada")
+  theWidget("ada_minsplit_spinbutton")$setSensitive(mtype=="ada")
+  theWidget("ada_complexity_label")$setSensitive(mtype=="ada")
+  theWidget("ada_cp_spinbutton")$setSensitive(mtype=="ada")
+  theWidget("ada_xval_label")$setSensitive(mtype=="ada")
+  theWidget("ada_xval_spinbutton")$setSensitive(mtype=="ada")
+  theWidget("ada_list_button")$setSensitive(mtype=="ada")
+  theWidget("ada_draw_button")$setSensitive(mtype=="ada")
+  theWidget("ada_draw_spinbutton")$setSensitive(mtype=="ada")
+  theWidget("ada_continue_button")$setSensitive(mtype=="ada")
+
+  theWidget("ada_learningrate_label")$setSensitive(mtype=="xgb")
+  theWidget("ada_learningrate_spinbutton")$setSensitive(mtype=="xgb")
+  theWidget("ada_nthread_label")$setSensitive(mtype=="xgb")
+  theWidget("ada_nthread_spinbutton")$setSensitive(mtype=="xgb")
+  theWidget("ada_objective_label")$setSensitive(mtype=="xgb")
+  theWidget("ada_objective_combobox")$setSensitive(mtype=="xgb")
+}
+
+on_model_boost_ada_radiobutton_toggled <- function(button)
+{
+  if (button$getActive())
+  {
+    theWidget("model_ada_builder_label")$setText(crv$ADA)
+    setADAOptions(crv$ADA)
+  }
+}
+
+on_model_boost_xgb_radiobutton_toggled <- function(button)
+{
+  if (button$getActive())
+  {
+    theWidget("model_ada_builder_label")$setText(crv$XGB)
+    setADAOptions(crv$XGB)
+  }
+}
+
 # Model -> Linear
 #
 # When any of the regression radion buttons change then ensure the
@@ -234,23 +280,37 @@ on_evaluate_model_checkbutton_toggled <- function(button)
 
 commonName <- function(mtype)
 {
-  name.map <- data.frame(ada=Rtxt("Ada Boost"),
-                         arules=Rtxt("Association Rules"),
-                         biclust=Rtxt("BiCluster"),
-                         cforest=Rtxt("Random Forest"),
-                         ctree=Rtxt("Conditional Tree"),
-                         ewkm=Rtxt("Entropy Weighted KMeans"),
-                         hclust=Rtxt("Hierarchical"),
-                         kmeans=Rtxt("KMeans"),
-                         rf=Rtxt("Random Forest"),
-                         rpart=Rtxt("Decision Tree"),
-                         svm=Rtxt("SVM"),
-                         ksvm=Rtxt("SVM"),
-                         glm=Rtxt("Linear"),
-                         linear=Rtxt("Linear"),
-                         multinom=Rtxt("Neural Net"),
-                         nnet=Rtxt("Neural Net"),
-                         survival=Rtxt("Survival"))
+  name.map <- data.frame(
+    
+    # "ada" should really be "boost" eventually. Check the actual
+    # model class and return the apporiate common name
+    
+    ada=ifelse(!is.null(crs$ada) || "xgb.Booster" %in% class(crs$ada),
+               Rtxt("Extreme Boost"), Rtxt("Ada Boost")),
+
+    arules=Rtxt("Association Rules"),
+    biclust=Rtxt("BiCluster"),
+    boost=Rtxt("Boosted Trees"),
+    cforest=Rtxt("Random Forest"),
+    ctree=Rtxt("Conditional Tree"),
+    ewkm=Rtxt("Entropy Weighted KMeans"),
+    hclust=Rtxt("Hierarchical"),
+    kmeans=Rtxt("KMeans"),
+    rf=Rtxt("Random Forest"),
+    rpart=Rtxt("Decision Tree"),
+    rxdtree=Rtxt("Microsoft Decision Tree"),
+    rxdforest=Rtxt("Microsoft Random Forest"),
+    svm=Rtxt("SVM"),
+    ksvm=Rtxt("SVM"),
+    glm=Rtxt("Linear"),
+    linear=Rtxt("Linear"),
+    rxbtrees=Rtxt("Microsoft Boosted Trees"),
+    rxGLM=Rtxt("Big Data Linear"),
+    multinom=Rtxt("Neural Net"),
+    nnet=Rtxt("Neural Net"),
+    survival=Rtxt("Survival"),
+    xgb=Rtxt("Extreme Boost"))
+  
   return(as.character(name.map[[mtype]]))
 }
 
@@ -348,7 +408,7 @@ existsPredictiveModel <- function()
 {
   # TRUE if there is a predictive model as distinct from a descriptive
   # model.
-  return(! is.null(listBuiltModels(c(crv$KMEANS, crv$HCLUST, crv$APRIORI))))
+  return(not.null(listBuiltModels(c(crv$KMEANS, crv$HCLUST, crv$APRIORI))))
 }
 
 noModelAvailable <- function(model, model.class)
@@ -474,13 +534,25 @@ executeModelTab <- function()
 
   start.time <- Sys.time()
 
-  if (build.all || currentModelTab() == crv$RPART)
+  #-- DECISION TREE MODEL -------------------------------------------------
+
+  # TODO: Tidy the logic as with the BOOST code.
+  
+  if (currentModelTab() == crv$RPART || build.all)
   {
     if (theWidget("rpart_build_radiobutton")$getActive())
     {
       setStatusBar(sprintf(Rtxt("Building %s model ..."), commonName(crv$RPART)))
 
-      if (theWidget("model_tree_ctree_radiobutton")$getActive())
+      if (not.null(crs$xdf))
+      {
+        if (executeModelRxDTree())
+          theWidget("evaluate_rpart_checkbutton")$setActive(TRUE)
+        else
+          setStatusBar(sprintf(Rtxt("Building %s model ... failed."),
+                               commonName("rxdtree")))
+      }
+      else if (theWidget("model_tree_ctree_radiobutton")$getActive())
       {
         if (executeModelCTree())
           theWidget("evaluate_rpart_checkbutton")$setActive(TRUE)
@@ -519,35 +591,72 @@ executeModelTab <- function()
       
     }
   }
-  if ((binomialTarget() && build.all)
-      || currentModelTab() == crv$ADA)
+
+  #-- BOOSTED MODEL -------------------------------------------------------
+  
+  if (currentModelTab() == crv$ADA || (binomialTarget() && build.all))
   {
-    setStatusBar(sprintf(Rtxt("Building %s model ..."), commonName(crv$ADA)))
-    crs$ada <-
-      buildModelAda(formula,
-                    dataset,
-                    tv=theWidget("ada_textview"),
-                    maxdepth=theWidget("ada_maxdepth_spinbutton")$getValue(),
-                    minsplit=theWidget("ada_minsplit_spinbutton")$getValue(),
-                    cp=theWidget("ada_cp_spinbutton")$getValue(),
-                    xval=theWidget("ada_xval_spinbutton")$getValue(),
-                    ntree=theWidget("ada_ntree_spinbutton")$getValue())
+    # Boosted Model Selected
+    
+    mname <- commonName(crv$BOOST) # Model name.
+    
+    setStatusBar(sprintf(Rtxt("Building %s model ..."), mname))
+      
+    if (not.null(crs$xdf))
+    {
+      # Microsoft R Boosted Trees
+      
+      executeModelRxBTrees()
+    }
+    else if (theWidget("model_boost_ada_radiobutton")$getActive())
+    {
+      # Adaptive Boosting
+      
+      executeModelAda(dataset, formula)
+    }
+    else if (theWidget("model_boost_xgb_radiobutton")$getActive())
+    {
+      # Extreme Gradient Boosting
+      
+      executeModelXGB(dataset, formula)
+    }
+
     if (not.null(crs$ada))
     {
+      # Model build succeeded so set the status and enable the
+      # appropriate buttons in the Model -> Boost tab.
+      
+      setStatusBar(sprintf(Rtxt("%s model build completed."), mname))
       showModelAdaExists()
       theWidget("evaluate_ada_checkbutton")$setActive(TRUE)
     }
     else
-      setStatusBar(sprintf(Rtxt("Building %s model ... failed."), commonName(crv$ADA)))
+    {
+      # Model build failed so simply report this in the status bar.
+      
+      setStatusBar(sprintf(Rtxt("Building %s model ... failed."), mname))
+    }
   }
-
+ 
+  #-- RANDOM FOREST MODEL -------------------------------------------------
+  
+  # TODO: Tidy the logic as with the BOOST code.
+  
   if (build.all || currentModelTab() == crv$RF)
   {
     setStatusBar(sprintf(Rtxt("Building %s model ..."), commonName(crv$RF)))
-    if (executeModelRF(traditional=theWidget("model_rf_traditional_radiobutton")$
-                       getActive(),
-                       conditional=theWidget("model_rf_conditional_radiobutton")$
-                       getActive()))
+    if (not.null(crs$xdf))
+    {
+      if (executeModelRxDForest())
+        theWidget("evaluate_rf_checkbutton")$setActive(TRUE)
+      else
+        setStatusBar(sprintf(Rtxt("Building %s model ... failed."),
+                             commonName("rxdforest")))
+    }
+    else if (executeModelRF(traditional=theWidget("model_rf_traditional_radiobutton")$
+                            getActive(),
+                            conditional=theWidget("model_rf_conditional_radiobutton")$
+                            getActive()))
       theWidget("evaluate_rf_checkbutton")$setActive(TRUE)
     else
       setStatusBar(sprintf(Rtxt("Building %s model ... failed."), commonName(crv$RF)))
@@ -565,8 +674,16 @@ executeModelTab <- function()
   }
   if (build.all || currentModelTab() == crv$GLM)
   {
-    setStatusBar(sprintf(Rtxt("Building %s model ..."), commonName(crv$GLM)))
-    if (executeModelGLM())
+    setStatusBar(sprintf(Rtxt("Building %s model ..."), commonName(crv$GLM)))    
+    if (not.null(crs$xdf))
+    {
+      if (executeModelRxGlm())
+        theWidget("evaluate_glm_checkbutton")$setActive(TRUE)
+      else
+        setStatusBar(sprintf(Rtxt("Building %s model ... failed."),
+                             commonName("rxglm")))
+    }
+    else if (executeModelGLM())
       theWidget("evaluate_glm_checkbutton")$setActive(TRUE)
     else
       setStatusBar(sprintf(Rtxt("Building %s model ... failed."), commonName(crv$GLM)))
@@ -624,326 +741,6 @@ executeModelTab <- function()
   }
 }
 
-#----------------------------------------------------------------------
-#
-# MODEL GLM
-#
-
-executeModelGLM <- function()
-{
-  # Initial setup. 
-  
-  TV <- "glm_textview"
-  mtype <- "linear"
-
-  # Obtain the family
-
-  if (theWidget("glm_linear_radiobutton")$getActive())
-    family <- "Linear"
-  else if (theWidget("glm_gaussian_radiobutton")$getActive())
-    family <- "Gaussian"
-  else if (theWidget("model_linear_poisson_radiobutton")$getActive())
-    family <- "Poisson"
-  else if (theWidget("glm_logistic_radiobutton")$getActive())
-    family <- "Logistic"
-  else if (theWidget("model_linear_probit_radiobutton")$getActive())
-    family <- "Probit"
-  else if (theWidget("glm_multinomial_radiobutton")$getActive())
-    family <- "Multinomial"
-  
-  # Build the formula for the model. 080719 If the user has requested
-  # a numeric target and the target is actually a factor, then convert
-  # to a numeric, else the algorithms complain.
-
-  if (family %in% c("Linear", "Gaussian", "Poisson")
-      && "factor" %in% class(crs$dataset[[crs$target]]))
-    frml <- sprintf("as.numeric(%s) ~ .", crs$target)
-  else
-    frml <- paste(crs$target, "~ .")
-
-  # List, as a string, the variables to be included. 
-  
-  included <- "c(crs$input, crs$target)" # 20110102 getIncludedVariables()
-  
-  # Some convenience booleans.
-
-  sampling  <- not.null(crs$train)
-  including <- not.null(included)
-  subsetting <- sampling || including
-  
-  startLog(Rtxt("Regression model"))
-
-  if (family == "Logistic" || family == "Probit")
-  {
-    # For a categoric variable we usually default to assuming
-    # proprtions data, and so we perform logistic regression, which
-    # uses a binomial distribution and a logit link function. However,
-    # the user could eventually choose a different distriubtion/link
-    # pair.
-    #
-    # If we have a binary response it may be that we might consider
-    # using a loglog link rather than a logit link.
-
-    model.cmd <- paste("crs$glm <- glm(", frml, ",\n    data=crs$dataset",
-                       if (subsetting) "[",
-                       if (sampling) "crs$train",
-                       if (subsetting) ", ",
-                       if (including) included,
-                       if (subsetting) "]",
-                       sprintf(',\n    family=binomial(link="%s")',
-                               ifelse(family=="Probit", "probit", "logit")),
-                       #", na.action=na.pass",
-                       if (! is.null(crs$weights))
-                              sprintf(",\n    weights=(%s)%s",
-                                      crs$weights,
-                                      ifelse(sampling, "[crs$train]", "")),
-                       ")", sep="")
-
-    # In addition to the default summary, add the chi-square test of
-    # the difference between the null model and the current model as
-    # presented in http://www.ats.ucla.edu/stat/R/dae/probit.htm.
-
-    
-    
-    summary.cmd <- paste("print(summary(crs$glm))",
-                         paste('cat(sprintf("Log likelihood: %.3f (%d df)\\n",',
-                               '            logLik(crs$glm)[1],',
-                               '            attr(logLik(crs$glm), "df")))', sep="\n"),
-                         paste('cat(sprintf("Null/Residual deviance difference:',
-                               '%.3f (%d df)\\n",'),
-                         '            crs$glm$null.deviance-crs$glm$deviance,',
-                         '            crs$glm$df.null-crs$glm$df.residual))',
-                         'cat(sprintf("Chi-square p-value: %.8f\\n",',
-                         '            dchisq(crs$glm$null.deviance-crs$glm$deviance,',
-                         '                   crs$glm$df.null-crs$glm$df.residual)))',
-                         'cat(sprintf("Pseudo R-Square (optimistic): %.8f\\n",',
-                         '             cor(crs$glm$y, crs$glm$fitted.values)))',
-                         "cat('\\n==== ANOVA ====\\n\\n')",
-                         'print(anova(crs$glm, test="Chisq"))',
-                         'cat("\\n")',
-                         sep="\n")
-  }
-  else if (family == "Linear")
-  {
-
-    # For a numeric target we expect to produce the usual linear
-    # model. We could use glm to generate the model using the gaussian
-    # distribution and the identity link function. This will produce
-    # the same model as lm. But lm is faster (glm is an iterative
-    # algorithm) and it also produces the R squared stats, so we use
-    # lm.
-    
-    model.cmd <- paste("crs$glm <- lm(", frml, ", data=crs$dataset",
-                       if (subsetting) "[",
-                       if (sampling) "crs$train",
-                       if (subsetting) ",",
-                       if (including) included,
-                       if (subsetting) "]",
-                       if (! is.null(crs$weights))
-                       sprintf(",\n    weights=(%s)%s",
-                               crs$weights,
-                               ifelse(sampling, "[crs$train]", "")),
-                      ")", sep="")
-
-    summary.cmd <- paste("print(summary(crs$glm))",
-                         "cat('==== ANOVA ====\n\n')",
-                         "print(anova(crs$glm))",
-                         'print("\n")',
-                         sep="\n")
-  }
-  else if (family == "Gaussian")
-  {
-    # Whilst this is a less efficient equivalent of the Linear model
-    # using lm, it is identified that some users perceive value in
-    # having both lm and glm options for numeric regression. This uses
-    # a gaussian distribution and an identity link function.
-
-    model.cmd <- paste("crs$glm <- glm(", frml, ", data=crs$dataset",
-                       if (subsetting) "[",
-                       if (sampling) "crs$train",
-                       if (subsetting) ",",
-                       if (including) included,
-                       if (subsetting) "]",
-                       ", family=gaussian(identity)",
-                       ")", sep="")
-
-    summary.cmd <- paste("print(summary(crs$glm))",
-                         "cat('==== ANOVA ====\n\n')",
-                         "print(anova(crs$glm))",
-                         'print("\n")',
-                         sep="\n")
-  }
-  else if (family == "Poisson")
-  {
-    # 080912 Added
-
-    model.cmd <- paste("crs$glm <- glm(", frml, ", data=crs$dataset",
-                       if (subsetting) "[",
-                       if (sampling) "crs$train",
-                       if (subsetting) ",",
-                       if (including) included,
-                       if (subsetting) "]",
-                       ", family=poisson(log)",
-                       ")", sep="")
-
-    summary.cmd <- paste("print(summary(crs$glm))",
-                         "cat('==== ANOVA ====\n\n')",
-                         "print(anova(crs$glm))",
-                         'print("\n")', sep="\n")
-  }
-  else if (family == "Multinomial")
-  {
-    lib.cmd <-  "library(nnet, quietly=TRUE)"
-    if (! packageIsAvailable("nnet", Rtxt("build a mulitnomial model"))) return(FALSE)
-    appendLog(Rtxt("Build a multinomial model using the nnet package."), lib.cmd)
-    eval(parse(text=lib.cmd))
-
-    car.available <- TRUE
-    lib.cmd <- "library(car, quietly=TRUE)"
-    if (! packageIsAvailable("car", Rtxt("use Anova to evaluate a mulitnomial model")))
-      car.available <- FALSE
-    else
-    {
-      appendLog(Rtxt("Summarise multinomial model using Anova from the car package."), lib.cmd)
-      eval(parse(text=lib.cmd))
-    }
-    
-    model.cmd <- paste("crs$glm <- ",
-                       "multinom",
-                       "(", frml, ", data=crs$dataset",
-                       if (subsetting) "[",
-                       if (sampling) "crs$train",
-                       if (subsetting) ",",
-                       if (including) included,
-                       if (subsetting) "]",
-                       ", trace=FALSE, maxit=1000",
-                       ")", sep="")
-
-    summary.cmd <- paste("rattle.print.summary.multinom(summary(crs$glm,",
-                         "                              Wald.ratios=TRUE))",
-                         paste('cat(sprintf("Log likelihood: %.3f (%d df)\n",',
-                               'logLik(crs$glm)[1], attr(logLik(crs$glm), "df")))'),
-                         paste('if (is.null(crs$glm$na.action)) omitted <- TRUE',
-                               'else omitted <- -crs$glm$na.action'),
-                         paste('cat(sprintf("Pseudo R-Square: %.8f\n\n",',
-                               'cor(apply(crs$glm$fitted.values, 1, ',
-                               'function(x) which(x == max(x))),\n',
-                               'as.integer(crs$dataset',
-                               ifelse(sampling, '[crs$train,]', ''),
-                               '[omitted,]$',
-                               crs$target, '))))\n', sep=""),
-                         "cat('==== ANOVA ====\n')",
-                         "print(Anova(crs$glm))",
-                         'print("\n")',
-                         sep="\n")
-
-  }
-  
-  # Build the model.
-
-  appendLog(Rtxt("Build a Regression model."),
-            model.cmd, sep="")
-  start.time <- Sys.time()
-  result <- try(eval(parse(text=model.cmd)), silent=TRUE)
-  if (inherits(result, "try-error"))
-  {
-    if (any(grep("too many (.*) weights", result)))
-    {
-      find.num.weights <- regexpr('\\([0-9]*\\)', result)
-      num.weights <- substr(result, find.num.weights,
-                            find.num.weights + attr(find.num.weights, "match.length")-1)
-      errorDialog(sprintf(Rtxt("The Multinomial model build has failed,",
-                               "with too many weights (%d)",
-                               "needing to be calculated.",
-                               "Perhaps consider reducing the",
-                               "number of categoric variables with",
-                               "unique values (if you have",
-                               "such variables in your input data)",
-                               "or perhaps treating the target",
-                               "variable as numeric and perform a",
-                               "numeric linear regression."),
-                          num.weights))
-      setTextview(TV)
-    }        
-    else if (any(grep("contrasts can be applied only to factors with 2", result)))
-    {
-      factors <- crs$input[sapply(crs$input, function(x)
-                                  is.factor(crs$dataset[[x]]))]
-      single <- factors[sapply(factors, function(x)
-                               length(levels(crs$dataset[[x]]))==1)]
-      one <- length(single)==1
-      errorDialog("It appears that", ifelse(one, "a", "some"),
-                  "categoric input",
-                  ifelse(one, "variable is", "variables are"), "constant.",
-                  "The regression model algorithm can not handle such",
-                  ifelse(one, "a variable.", "variables."),
-                  "You may like to Ignore the",
-                  ifelse(one, "variable", "variables"),
-                  "through the Data tab:\n\n",
-                  paste(single, collapse=", "))
-      setTextview(TV)
-    }
-    else
-      errorDialog(Rtxt("The regression model appears to have failed.",
-                       "The error message was:"),
-                  result, crv$support.msg)
-    return(FALSE)
-  }
-  
-  # Summarise the model.
-
-  appendLog(sprintf(Rtxt("Generate a textual view of the %s model."),
-                    commonName(crv$GLM)), summary.cmd)
-  
-  resetTextview(TV)
-  setTextview(TV, sprintf(Rtxt("Summary of the %s %s model",
-                               "(built using %s):\n"),
-                          family, "Regression",
-                          ifelse(family == "Linear", "lm",
-                                 ifelse(family == "Multinomial", "multinom", "glm"))),
-              ifelse(any(is.na(coef(crs$glm))),
-                     paste("\n***Note*** Singularities were found in the modeling
-and are indicated by an NA in the following table.
-This is often the case when variables are linear
-combinations of other variables, or the variable
-has a constant value.  These variables will be ignored
-when using the model to score new data and will not be
-included as parameters in the exported scoring routine.\n",
-                           sep="\n"), ""),
-              collectOutput(summary.cmd))
-
-# 090223 Got it working above - just wrap a print around it?
-#
-#  if (family == "Multinomial" && car.available)
-#  {
-#    # Couldn't get this working within the summary.cmd
-#    appendTextview(TV, paste("\n\n",
-#                             collectOutput("Anova(crs$glm)", use.print=TRUE)),
-#                   tvsep=FALSE)
-#  }
-  
-  
-  if (sampling) crs$smodel <- union(crs$smodel, crv$GLM)
-
-  # Enable the plot button
-
-  if (family == "Multinomial")
-    theWidget("model_linear_plot_button")$setSensitive(FALSE)
-  else
-    theWidget("model_linear_plot_button")$setSensitive(TRUE)
-  
-  # Finish up.
-  
-  time.taken <- Sys.time()-start.time
-
-  reportTimeTaken(TV, time.taken,
-                  msg=paste(sprintf(Rtxt("The %s model has been built."),
-                    commonName(mtype)),
-                    ifelse(any(is.na(coef(crs$glm))),
-                           Rtxt("Singularities exist."), "")))
-  return(TRUE)
-}
-
 rattle.print.summary.multinom <- function (x, digits = x$digits, ...) 
 {
   # All I want is to add "n=XXX" here!!!
@@ -995,7 +792,7 @@ exportRegressionModel <- function()
 
   # Generate appropriate code.
   
-  if (! is.null(crs$weights))
+  if (not.null(crs$weights))
     wt <- gsub("^\\(|\\)$", "",
                gsub("crs\\$dataset\\$|\\[.*\\]", "",
                     capture.output(print(crs$weights))))
@@ -1177,7 +974,7 @@ executeModelSVM <- function()
 
   # Replicate rows according to the integer weights variable.
   
-  if(! is.null(crs$weights))
+  if(not.null(crs$weights))
     dataset <- paste(dataset,
                      "[rep(row.names(",
                      dataset,
@@ -1313,13 +1110,13 @@ exportSVMModel <- function()
 
 #  if (get.extension(save.name) == "") save.name <- sprintf("%s.xml", save.name)
 
-  if (! is.null(crs$weights))
+  if (not.null(crs$weights))
     wt <- gsub("^\\(|\\)$", "",
                gsub("crs\\$dataset\\$|\\[.*\\]", "",
                     capture.output(print(crs$weights))))
   
   pmml.cmd <- paste('pmml(crs$ksvm, dataset=crs$dataset',
-                    if (! is.null(crs$weights))
+                    if (not.null(crs$weights))
                     paste(', weights=', wt, sep=""),
                     ')', sep="")
   appendLog(Rtxt("Export a SVM model as PMML."),
